﻿Imports System.ComponentModel
Imports System.Drawing
Imports System.IO
Imports System.Windows.Forms

<System.ComponentModel.DefaultBindingProperty("PicturePath")>
Public Class PathBoundPictureBox
    Public Event PicturePathChanged(ByVal Path As String)
    Private BorderColorValue As Color = Color.Black
    Private Path As String = ""
    Private CenteredImageValue As Boolean = False

    Public Sub New()
        InitializeComponent()
        Me.SetStyle(ControlStyles.OptimizedDoubleBuffer Or ControlStyles.ResizeRedraw Or ControlStyles.UserPaint, True)
        Me.UpdateStyles()
    End Sub

    <Bindable(True), RefreshProperties(RefreshProperties.All), SettingsBindable(True), DefaultValue("")>
    Public Property PicturePath() As String
        Get
            Return Path
        End Get
        Set(ByVal value As String)
            If Path <> value Then
                Path = value
                RaiseEvent PicturePathChanged(Path)
                Me.Invalidate()
            End If
        End Set
    End Property

    <Category("Appearance")>
    Public Property BorderColor As Color
        Get
            Return BorderColorValue
        End Get
        Set(value As Color)
            BorderColorValue = value
            Me.Invalidate()
        End Set
    End Property

    <Category("Appearance")>
    Public Property CenteredImage As Boolean
        Get
            Return CenteredImageValue
        End Get
        Set(value As Boolean)
            CenteredImageValue = value
            Me.Invalidate()
        End Set
    End Property

    Private Function PathToBitmap(ByVal VPath As String) As Image
        Dim PicImage As Image = Nothing
        If VPath <> "" Then
            If File.Exists(Path) Then
                Using fs As New FileStream(Path, FileMode.Open, FileAccess.Read)
                    PicImage = Image.FromStream(fs)
                End Using
            End If
        End If
        Return PicImage
    End Function

    Private Function CalculateRect(ByVal imgSize As Size, ByVal SurfaceSize As Size) As Rectangle
        Dim originalWidth As Integer = imgSize.Width
        Dim originalHeight As Integer = imgSize.Height
        Dim percentWidth As Single = CSng(SurfaceSize.Width / originalWidth)
        Dim percentHeight As Single = CSng(SurfaceSize.Height / originalHeight)
        Dim percent As Single
        Select Case percentHeight < percentWidth
            Case True
                percent = percentHeight
            Case False
                percent = percentWidth
        End Select
        Dim newWidth As Integer = CInt(originalWidth * percent)
        Dim newHeight As Integer = CInt(originalHeight * percent)
        Dim startLeft As Integer = CInt((SurfaceSize.Width - newWidth) / 2)
        Dim startTop As Integer = CInt((SurfaceSize.Height - newHeight) / 2)
        Return New Rectangle(startLeft, startTop, newWidth, newHeight)
    End Function

    Protected Overrides Sub OnPaint(e As PaintEventArgs)
        Dim BorderBrush As New SolidBrush(BorderColorValue)
        Dim BorderPen As New Pen(BorderBrush)
        Dim DrawImage As Image = PathToBitmap(Path)
        With e.Graphics
            If DrawImage IsNot Nothing Then
                Select Case CenteredImageValue
                    Case True
                        .DrawImage(DrawImage, CalculateRect(DrawImage.Size, Me.Size))
                    Case False
                        .DrawImage(DrawImage, New Rectangle(1, 1, Me.Width - 2, Me.Height - 2))
                End Select
                DrawImage.Dispose()
            End If
            .DrawRectangle(BorderPen, New Rectangle(0, 0, Me.Width - 1, Me.Height - 1))
        End With
        Validate()
        BorderPen.Dispose()
        BorderBrush.Dispose()

    End Sub
End Class